;Software License Agreement                                         
;                                                                    
;The software supplied herewith by Microchip Technology             
;Incorporated (the "Company") is intended and supplied to you, the  
;Companys customer, for use solely and exclusively on Microchip    
;products. The software is owned by the Company and/or its supplier,
;and is protected under applicable copyright laws. All rights are   
;reserved. Any use in violation of the foregoing restrictions may   
;subject the user to criminal sanctions under applicable laws, as   
;well as to civil liability for the breach of the terms and         
;conditions of this license.                                        
;                                                                    
;THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,  
;WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED  
;TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A       
;PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,  
;IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR         
;CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.   
;**********************************************************************
;
;	Filename:	triac.asm
;	Date:		September 29, 2004
;	Version:	--
;
;	Author:		Reston Condit
;	Company:	Microchip Technology Inc.
;                                                                     
;                                                                     
;**********************************************************************
;	Description:
;	This code implements a triac control routine for varying power to a
;	heating element like an electric range.  Input is provided by the 
;	user via a potentiometer.  This is the source code for AN958.  
; 	
;**********************************************************************
;	Revision History: none
;
;**********************************************************************

	#include <p10F206.inc>        ; processor specific variable definitions

	errorlevel  -302              ; suppress message 302 from list file

	__CONFIG   _CP_OFF & _MCLRE_OFF & _WDT_ON & _IntRC_OSC 

; '__CONFIG' directive is used to embed configuration word within .asm file.
; The lables following the directive are located in the respective .inc file.
; See data sheet for additional information on configuration word settings.

;************************** VARIABLE DEFINITIONS ******************************

      cblock	0x08
		OnCycles			; number of cycles on per total cycles 
							;  1 half wave = 1 cycle
							;  120 cycles per second (60Hz times 2)  
		CycleCounter		; cycle counter
		Toggler				; used to detect transistion on zero cross
							; this register "toggles" back and forth so that
							;  a zero cross is detected
		DischargeTimer		; counts the number of cycles it takes to discharge
							;  the potentiometer RC circuit
		RAMTRIS				; this is a baseline part so I have to create my 
							;  own tris register in RAM to keep track of my
							;  input and output pins (very important!)
		AutoSDTimerL		; counters for autoshutdown
		AutoSDTimerH
		AutoSDTimerZ
		Flag				; contains misc flags
      endc

;*************************** DEFINE STATEMENTS ********************************

;; pin definitions
#define	LED	 	GPIO,0
#define	POT 	GPIO,1
#define	TRIAC	GPIO,2
#define	ZEROX 	GPIO,3

;; masks for manipulating the POT and triac pins
#define	POT_OUTPUT		b'11111101'	; and with this to make pot pin an output
#define POT_INPUT		b'00000010'	; ior with this to make pot pin an input
#define	TRIAC_OUTPUT	b'11111011'	; and with this to make triac pin an output
#define	TRIAC_INPUT		b'00000100'	; ior with this to make triac pin an input

#define TOTALNUMCYCLES	.10
#define	SETPOINT1		.190
#define	SETPOINT2		.175
#define	SETPOINT3		.150
#define	SETPOINT4		.135
#define	SETPOINT5		.120
#define NUMONCYCLES1	.0
#define NUMONCYCLES2	.2
#define NUMONCYCLES3	.4
#define NUMONCYCLES4	.6
#define NUMONCYCLES5	.8
#define NUMONCYCLES6	.10

#define SHUTDOWNACTIVE	Flag,0
#define	POTSTATE		Flag,1

;****************************** START OF CODE *********************************
	org 	0x00
	movlw	0x40 	;****** REMOVE THIS LINE FOR PRODUCTION ****** 	
	movwf	OSCCAL		; load the factory oscillator calibration value

;******************************************************************************
; InitializeSFRs - Initialize Special Function Registers
;
;******************************************************************************
InitializeSFRs
	movlw	b'01111000' ; turn on comparator
	movwf	CMCON0
	movlw	b'00001010'	; Outputs: LED and Triac pins
	movwf	RAMTRIS
	tris	GPIO
	clrf	GPIO		; turn on LED and Triac
	movlw	b'01000100' ; 1:32TMR0 prescaler: 1E-6 x 256 x 32= 8.192ms, weak pullups disabled
	option

;******************************************************************************
; InitializeVariables - Initialize Program Variables
;
;******************************************************************************
InitializeVariables
	movlw	0x08			; initialize Toggler to trigger on positive half-wave first
	movwf	Toggler
	movlw	0				; number of cycles on out of total cycles (120 cycles per second)
	movwf	OnCycles	
	clrf	CycleCounter	; cycle counter
	clrf	AutoSDTimerL	; Initialize Auto Shutdown Timer (set for 2 hours)
	clrf	AutoSDTimerH
	movlw	.13			
	movwf	AutoSDTimerZ
	clrf	Flag

;******************************************************************************
; Loop - Main program loop
;   - wait for zero cross pin to toggle
;	- check to see if triac should conduct during this half-cycle
;	  * if it should conduct then pulse the gate of the triac
;	- if in the positive half-cycle, then call PotReadState0 to charge capacitor
;	- if in the negative half-cycle, then call PotReadState1 to time the discharge of the RC circuit
;
;******************************************************************************
Loop
	clrwdt					; clear Watch Dog Timer
	movf	GPIO,w			; mask ZEROX bit	
	andlw	0x08
	xorwf	Toggler,w		; check against toggler (did ZEROX make a transition?)
	btfss	STATUS,Z
	goto	Loop			; no, then loop
	movlw	0x08			; yes, then toggle toggler and move on
	xorwf	Toggler,f
	clrf	TMR0			; clear the timer at zero cross
	movf	OnCycles,w		; if this is an on cycle then turn on triac		
	subwf	CycleCounter,w	; otherwise leave triac off and continue
	btfsc	STATUS,C
	goto	EndLoop
	bsf		TRIAC			; pulse Triac gate to turn it on

EndLoop
	incf	CycleCounter,f	; increment cycle counter
	movlw	TOTALNUMCYCLES	; if total cycles reached then clear cycle counter
	subwf	CycleCounter,w
	btfsc	STATUS,C
	clrf	CycleCounter
	btfss	POTSTATE		
	goto	PotReadState0	; charge capacitor in potentiometer read circuit
	btfsc	POTSTATE		
	goto	PotReadState1	; time decay of capacitor
	goto	Loop

;******************************************************************************
; PotReadState0 - Charge the capacitor in the Potentiometer RC circuit
;	- turn off comparator
;	- charge Poteniometer RC capacitor
;	- turn off triac gate after 2ms
;	- toggle potentiometer read state 
;
;******************************************************************************
PotReadState0
	bcf 	CMCON0,CMPON	; turn off comparitor
	movf	RAMTRIS,w
	andlw	POT_OUTPUT		; make POT pin an output
	movwf	RAMTRIS
	tris	GPIO			
	bsf		POT				; charge cap
	bsf		POTSTATE		; ready for next state
	clrf	DischargeTimer	; initialize capacitor discharge timer

	clrwdt					; clear Watch Dog Timer
	btfss	TMR0,6			; wait 2ms and turn off triac
	goto	$-2
	bcf 	TRIAC			; turn off Triac Gate
	goto	CheckAutoShutDownTimer

;******************************************************************************
; PotReadState0 - Release capacitor in potentiometer read circuit and measure
;					its decay time
;	- configure comparator for read
;	- turn off triac gate after 2ms
;	- check for comparator trip
;	- once tripped, save TMR0 value
;	- based on this value set the number of on cycles
;
;******************************************************************************
PotReadState1
	bsf 	CMCON0,CMPON	; turn on comparator
	movf	RAMTRIS,w
	iorlw	POT_INPUT		; make pot pin an input, RC circuit starts to discharge
	movwf	RAMTRIS
	tris	GPIO
	bcf 	POTSTATE		; ready for next state

	clrwdt					; clear Watch Dog Timer
	btfss	TMR0,6			; wait 2ms and turn off triac
	goto	$-2
	bcf 	TRIAC			; turn off Triac Gate

CheckForComparatorTrip
	clrwdt					; clear Watch Dog Timer
	btfsc	CMCON0,CMPOUT
	goto	CheckForComparatorTrip
	movf	TMR0,w
	movwf	DischargeTimer

LookupTable
	movlw	SETPOINT1		; check Discharge Time against first setpoint	
	subwf	DischargeTimer,w
	btfss	STATUS,C
	goto	NextLookup1		; if measured time is lower then goto next setpoint comparison
	movlw	NUMONCYCLES1	; otherwise, set number of on cycles appropiately
	movwf	OnCycles
	bcf		LED				; Turn off LED
	bcf		SHUTDOWNACTIVE	; Reset SHUTDOWNACTIVE Flag if POT is turned to off
	clrf	AutoSDTimerL	; Initial Auto Shutdown Timer when burner is off
	clrf	AutoSDTimerH
	movlw	.13
	movwf	AutoSDTimerZ
	goto	CheckAutoShutDownTimer
NextLookup1
	btfsc	SHUTDOWNACTIVE
	goto	CheckAutoShutDownTimer
	bsf		LED				; Turn on LED
	movlw	SETPOINT2	
	subwf	DischargeTimer,w
	btfss	STATUS,C
	goto	NextLookup2
	movlw	NUMONCYCLES2
	movwf	OnCycles
	goto	CheckAutoShutDownTimer
NextLookup2
	movlw	SETPOINT3	
	subwf	DischargeTimer,w
	btfss	STATUS,C
	goto	NextLookup3
	movlw	NUMONCYCLES3
	movwf	OnCycles
	goto	CheckAutoShutDownTimer
NextLookup3
	movlw	SETPOINT4	
	subwf	DischargeTimer,w
	btfss	STATUS,C
	goto	NextLookup4
	movlw	NUMONCYCLES4
	movwf	OnCycles
	goto	CheckAutoShutDownTimer
NextLookup4
	movlw	SETPOINT5
	subwf	DischargeTimer,w
	btfss	STATUS,C
	goto	NextLookup5
	movlw	NUMONCYCLES5
	movwf	OnCycles
	goto	CheckAutoShutDownTimer
NextLookup5
	movlw	NUMONCYCLES6
	movwf	OnCycles
	goto	CheckAutoShutDownTimer

;******************************************************************************
; CheckAutoShutDownTimer - If the range has been on continuously for 2 hours
;							then shut it off
;	- auto shutdown timer overflows at 1 hour 58min and 12sec
;	- when it overflows, turn off triac and LED
;	- will reset when user interrupts power or turns dial back to no cycles on 
;		position
;******************************************************************************
CheckAutoShutDownTimer
	incfsz	AutoSDTimerL,f
	goto	Loop
	incfsz	AutoSDTimerH,f
	goto	Loop
	decfsz	AutoSDTimerZ,f
	goto	Loop

AutoShutDownTripped
	bsf		SHUTDOWNACTIVE		; set flag to indicate that auto-shutdown is active
	clrf	OnCycles			; turn off triac and status LED
	bcf		LED
	goto	Loop

	END                       	; directive 'end of program'
